rem Title: Cone/Frustum Flattener V1.06
rem Author: Peter Clifton
rem Date: 17th December 2001
rem Posted: cnews.corel.com

rem Newsgroup: corel.graphic_apps.draw9, corel.graphic_apps.draw-script
rem Large portions of this code: Copyright 1999 Alex Vakulenko
rem
rem This program was written in response to a post by
rem  Doug Von Gausig on the 14th November 2001 in the above
rem  Draw 9 newsgroup. I liked the subject so I investigated further.
rem
rem This program is based on information obtained from
rem  http://mathforum.org/dr.math/
rem  in particular the posts
rem  http://mathforum.org/dr.math/problems/blackburn12.9.96.html
rem  &
rem  http://mathforum.org/dr.math/problems/hartman.5.10.00.html
rem
rem Special thanks to Alex Vakulenko and his site at :-
rem  http://www.vakcer.com/oberon
rem  Without examination of his script resources this program would
rem  not have been possible.
rem
rem Revision history
rem V1.06
rem Tidy up construction and tell the user script has finished successfully.
rem V1.05
rem New routines to bypass using the arc/segment function during create ellipse routine
rem 	this is where precision was being lost, now much better, YES! Thanks to Nick
rem  Wilkinson (http://www.isocalc.com/) for pointing out the source of the problem)
rem V1.04 :-
rem	Fixed bug if units type changed after entering values
rem V1.03 :-
rem 	Added Glue Tab feature
rem V1.02 :-
rem 	Redefined PI constant
rem V1.01 :-
rem 	Added code to reverse radii if Base Radius = 0 and Top Radius > 0
rem V1.00 Created 20th November 2001

DECLARE SUB SetUnits(BYVAL u%)
DECLARE SUB CreateObjectx
DECLARE FUNCTION VConvert!(BYVAL v!,BYVAL dest%,BYVAL src%)

GLOBAL ShortUnits$(3),Units$(3),UnitSys%(4),Unit%, construct&(3)
ShortUnits$(1)="mm"
ShortUnits$(2)="in"
ShortUnits$(3)="cm"

Units$(1)="Millimetres"
Units$(2)="Inches"
Units$(3)="Centimetres"

UnitSys(1)=2
UnitSys(2)=1
UnitSys(3)=2
UnitSys(4)=7

GLOBAL bd!, td!, ch!,tempdiam!,majsh!,frussh!,misssh!,cangle#,pi#

BEGIN DIALOG OBJECT ConeSize 140, 155, "Cone/Frustum Flattener", SUB ConeSizeFunc
	TEXT  31, 9, 18, 8, .Text1, "Units"
	DDLISTBOX  52, 6, 55, 63, .Units
	GROUPBOX  10, 26, 120, 76, .GroupBox1, "Cone / Frustum Size"
	TEXT  15, 41, 42, 8, .Text2, "&Base Radius"
	SPINCONTROL  59, 39, 49, 14, .BaseRadius
	TEXT  113, 41, 13, 8, .Unit1, "&mm"
	TEXT  18, 63, 39, 9, .Text3, "&Top Radius"
	SPINCONTROL  59, 60, 49, 14, .TopRadius
	TEXT  113, 63, 13, 8, .Unit2, "&mm"
	TEXT  34, 84, 23, 8, .Text4, "&Height"
	SPINCONTROL  59, 82, 49, 14, .ConeHeight
	TEXT  113, 84, 13, 8, .Unit3, "&mm"
	CHECKBOX  49, 110, 43, 11, .GlueTab, "&Glue Tab"
	PUSHBUTTON  16, 129, 41, 17, .create, "Create"
	CANCELBUTTON  82, 129, 41, 17, .cancel
END DIALOG

SUB ConeSizeFunc(BYVAL ControlID%, BYVAL EventCode%)
SELECT CASE EventCode
	CASE 2
		SELECT CASE ControlID
			CASE ConeSize.Units.GETID()
				SetUnits ConeSize.Units.GETSELECT()
			CASE ConeSize.create.GETID()
				bd = ConeSize.BaseRadius.GETVALUE() * 2
				td = ConeSize.TopRadius.GETVALUE() * 2
				ch = ConeSize.ConeHeight.GETVALUE()
				CreateObjectX
		END SELECT
	END SELECT
END SUB

WITHOBJECT  "CorelDraw.Automation.9"
ConeSize.Units.SETARRAY Units$
ConeSize.Units.SETSELECT 1
ConeSize.BaseRadius.SETMINRANGE 0
ConeSize.BaseRadius.SETDOUBLEMODE TRUE
ConeSize.BaseRadius.SETINCREMENT 0.01
ConeSize.BaseRadius.SETPRECISION 3
ConeSize.TopRadius.SETMINRANGE 0
ConeSize.TopRadius.SETDOUBLEMODE TRUE
ConeSize.TopRadius.SETINCREMENT 0.01
ConeSize.TopRadius.SETPRECISION 3
ConeSize.ConeHeight.SETMINRANGE 0
ConeSize.ConeHeight.SETDOUBLEMODE TRUE
ConeSize.ConeHeight.SETINCREMENT 0.01
ConeSize.ConeHeight.SETPRECISION 3
ConeSize.GlueTab.SetThreeState FALSE

Unit%=1
pi#=atan(1)*4
SetUnits 1
ret&=DIALOG (ConeSize)
STOP


SUB SetUnits(BYVAL u%)
	ConeSize.Unit1.SETTEXT ShortUnits$(u)
	ConeSize.Unit2.SETTEXT ShortUnits$(u)
	ConeSize.Unit3.SETTEXT ShortUnits$(u)
	ConeSize.BaseRadius.SETVALUE VConvert(ConeSize.BaseRadius.GETVALUE(),u,Unit)
	ConeSize.TopRadius.SETVALUE VConvert(ConeSize.TopRadius.GETVALUE(),u,Unit)
	ConeSize.ConeHeight.SETVALUE VConvert(ConeSize.ConeHeight.GETVALUE(),u,Unit)
	Unit=u
END SUB

FUNCTION VConvert!(BYVAL v!,BYVAL dest%,BYVAL src%)
	IF src=1 THEN v=v/10
	v=LengthConvert(UnitSys(src),UnitSys(dest),v)
	IF dest=1 THEN v=v*10
	VConvert=v
END FUNCTION

SUB CreateObjectX
IF td = 0 and bd=0 or ch=0 or td=bd THEN
	MESSAGE "You cannot create a 2D plane cone or frustum with given dimensions"
	EXIT SUB
ENDIF
IF td <> 0 AND bd <> 0 THEN
	IF td > bd THEN
		tempdiam = td
		td=bd
		bd=tempdiam
	ENDIF
	frussh = VConvert(SQR((ch^2 + (bd-td)^2/4)), 4, Unit)
	majsh = VConvert(SQR(((bd * ch / (bd -td))^2) + (bd^2)/4), 4, Unit)
	misssh = majsh - frussh
	cangle = AngleConvert(2,1,(pi*bd)/SQR(((bd * ch / (bd -td))^2) + (bd^2)/4))
	rem new bit
		desty = sin(cangle*(pi/180))* (majsh+10000)
		destx = cos(cangle*(pi/180))* (majsh+10000)
		.BeginDrawCurve majsh+10000, 0
		.DrawCurveLineTo 0, 0
		.EndDrawCurve
		.AppendCurveLine 0,0,destx,desty
		construct&(1)=.GetObjectsCDRStaticID()
		.CreateEllipse -majsh, -majsh, majsh, majsh, 0, 0, FALSE
		construct&(2)=.GetObjectsCDRStaticID()
		.SelectObjectOfCDRStaticID construct&(1)
		.AppendObjectToSelection construct&(2)
		.TRIM FALSE,FALSE
		.BreakApart
		.UnselectAll
		.SelectNextObject FALSE
		construct&(1)=.GetObjectsCDRStaticID()
		.SelectNextObject FALSE
		.DeleteObject
	rem end newbit
	.CreateEllipse -misssh, -misssh, misssh, misssh, 0, 0, FALSE
	construct&(2)=.GetObjectsCDRStaticID()
	.SelectObjectOfCDRStaticID construct&(2)
	.AppendObjectToSelection construct&(1)
	IF ConeSize.GlueTab.GETVALUE() THEN
		.TRIM FALSE,TRUE
		construct&(1)=.GetObjectsCDRStaticID()
		.CreateEllipse -majsh, -majsh, majsh, majsh, -cangle*80000, 0, TRUE
		construct&(3)=.GetObjectsCDRStaticID()
		.SelectObjectOfCDRStaticID construct&(2)
		.AppendObjectToSelection construct&(3)
		.TRIM FALSE,FALSE
		.AppendObjectToSelection construct&(1)
		.Group
		.AlignToCenterOfPage 3,3
		.Ungroup
	ELSE
		.TRIM FALSE,FALSE
		.AlignToCenterOfPage 3,3
	ENDIF
ELSE
	IF bd = 0 THEN
		tempdiam=td
		td=bd
		bd=tempdiam
	ENDIF
	majsh = VConvert(SQR(((bd * ch / (bd -td))^2) + (bd^2)/4), 4, Unit)
	cangle = AngleConvert(2,1,(pi*bd)/SQR(((bd * ch / (bd -td))^2) + (bd^2)/4))
	rem new bit
	desty = sin(cangle*(pi/180))* (majsh+10000)
	destx = cos(cangle*(pi/180))* (majsh+10000)
	.BeginDrawCurve majsh+10000, 0
	.DrawCurveLineTo 0, 0
	.EndDrawCurve
	.AppendCurveLine 0,0,destx,desty
	construct&(1)=.GetObjectsCDRStaticID()
	.CreateEllipse -majsh, -majsh, majsh, majsh, 0, 0, FALSE
	construct&(2)=.GetObjectsCDRStaticID()
	.SelectObjectOfCDRStaticID construct&(1)
	.AppendObjectToSelection construct&(2)
	.TRIM FALSE,FALSE
	.BreakApart
	.UnselectAll
	.SelectNextObject FALSE
	construct&(1)=.GetObjectsCDRStaticID()
	.SelectNextObject FALSE
	.DeleteObject
	rem end new bit
	IF ConeSize.GlueTab.GETVALUE() THEN
		.CreateEllipse -majsh, -majsh, majsh, majsh, -cangle*80000, 0, TRUE
		.ConvertToCurves
		construct&(2)=.GetObjectsCDRStaticID()
		.SelectObjectOfCDRStaticID construct&(1)
		.AppendObjectToSelection construct&(2)
		.Group
		.AlignToCenterOfPage 3,3
		.Ungroup
	ELSE
		.SelectObjectOfCDRStaticID construct&(1)
		.AlignToCenterOfPage 3,3
	ENDIF

ENDIF

MESSAGE "2D View created"

END SUB

STOP

END WITHOBJECT

